function [X_iter,fun_all,smooth_iter]=ista(B,P,center,lambda,pars)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function implements Multilevel ISTA for solving the linear inverse
% problem with an orthogonal transform regulariser using cosine transformation 
% and "reflexive" boundary condition. This function performs computations 
% at the finest resolution, and it calls smooth_sqrt function to obtain
% coarse error correction. It is also possible to use this function
% recursively to obtain coarse error correction.
% 
% Based on the paper:
% - Panos Parpas, Duy V.N. Luong, Daniel Rueckert and Berc Rustem, 
% "A multilevel Proximal Algorithm for Large Scale Composite Convex Optimization" 
% Submitted to Mathematical Programming.
% -----------------------------------------------------------------------
% Further references:
% - Amir Beck and Marc Teboulle, 
% "A Fast Iterative Shrinkage-Threshold Algorithm for Linear Inverse Problems",
% SIAM Journal on Imaging Sciences.
% - P.C. Hansen, J.G Nagy, and D.P. O'Leary, 
% "Deblurring images: Matrices, spectra and filtering",
% Fundamentals of Algorithms. SIAM, Philadelphia 2006.
% -----------------------------------------------------------------------
% Copyright (2014): 
% Panos Parpas, Duy V.N. Luong, Daniel Rueckert and Berc Rustem
%
% MISTA is distributed under the terms of
% the GNU General Public License 2.0.
%
% Permission to use, copy, modify, and distribute this software for
% any purpose without fee is hereby granted, provided that this entire
% notice is included in all copies of any software which is or includes
% a copy or modification of this software and in all copies of the
% supporting documentation for such software.
% This software is being provided "as is", without any express or
% implied warranty. In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose."
% -----------------------------------------------------------------------
% ----------------------------------------------------------------------- 
% INPUT
%
% B             : The observed image which is blurred and noisy
% P             : point-spread-function PSF of the blurring operator
% center        : A vector of length 2 containing the center of the PSF
% lambda        : Regularisation parameter
% ** P & center for computation of sparse column/row blurring operators
% pars          : Parameters structure
% pars.MAXITER  : maximum number of iterations (Default = 50)
% pars.wave     : orthogonal transformation (@dct)
% pars.iwave    : orthogonal transformation (@idct)
% pars.fig      : if fig>0 show image at each iteration, 0 otherwise (7)
% pars.start    : starting image for optimisation
% pars.epsilon  : stopping condition, the percentage of F_k/F_0 (0.02)
% ** Without epsilon, program will run MAXITER iterations
% pars.kappa    : first condition to do coarse correction (0.49)
% pars.eta      : second condition to do coarse correction (1) 
% ** Smaller kappa & eta encourage more coarse correction
% -----------------------------------------------------------------------
% OUTPUT
% 
% X_iter        : Solution of the problem min |A(X)-B|^2 + lambda |Wx|_1
% fun_all       : History of function values at the finest resolution
% smooth_iter   : Record coarse correction iterations

% Assigning parameters according to pars and/or default values
flag=exist('pars','var');
if (flag && isfield(pars,'wave'))
    W=pars.wave;    
    WT=pars.iwave;    
else
    W=@dct;
    WT=@idct;
end
if (flag && isfield(pars,'start'))
    B_start=pars.start;    
else
    B_start = B;
end
MAXITER=pars.MAXITER;
fig=pars.fig;
BC = pars.BC;
W  = pars.wave;
WT = pars.iwave;
kappa=pars.kappa;    
eta=pars.eta;    
epsilon=pars.epsilon;    
%% Initialization for fine level
% computing the terms of the kronecker product
[m,n]=size(B);
Pbig=padPSF(P,[m,n]);
[Ar,Ac]=kronDecomp(Pbig,center,BC);
Ar(Ar<1e-10) = 0;
Ac(Ac<1e-10) = 0;
Ar = sparse(Ar);
Ac = sparse(Ac);
% computing the singular values of the Kronecker product of Ar and Ac
sr=svds(Ar);
sc=svds(Ac);
Sbig=sc*sr';
%The Lipschitz constant of the gradient of |A(X)-B|^2
L=2*max(max(abs(Sbig).^2)); 
% starting point
X_iter=B_start;
% compute first function value
t=sum(sum(abs(W(X_iter))));
fun_all=norm(Ac*X_iter*Ar'-B,'fro')^2+lambda*t;    

%% Parameters for coarse levels
% Set index for restriction operators
[m,n]=size(B);
M = m/2; N = n/2;
id1 = (1:2:m)';
id2 = (2:2:m)';
for i=1:(n/2)-1,
    id1 = [id1; (2*i*m+1:2:2*i*m+m)'];
    id2 = [id2; (2*i*m+2:2:2*i*m+m)'];    
end;
% scale down the image        
smallB = B(id1) + B(id2) + B(id1+m) + B(id2+m);    
smallB = reshape(smallB/4,[M,N]);    
smallPars = [];
smallPars.epsilon = 0.01;
% Compute the coarse col and row matrix 
R = ones(1,2);
R = kron(speye(M,M),R);
Ac_H = R*Ac*R'/2;
Ar_H = R*Ar*R'/2;
smooth_iter = [];

%% Start fine iterations
X_lastcoarse = rand(size(X_iter));

fprintf('********************MISTA-F **********************\n');
fprintf('#iter  funtion     kappa   eta -------------------\n');
fprintf('%4d  %10.5f\n',0,fun_all);
l = 0;
for iter=1:MAXITER    
    % Store the old value of the iterate and the t-constant
    X_old=X_iter;             
    % Gradient step    
    X_iter=X_old-2/L*Ac'*(Ac*X_old*Ar'-B)*Ar;    
    % Orthogonal transform
    WX=W(X_iter);
    % Soft thresholding 
    D=abs(WX)-lambda/(L);
    WX=sign(WX).*((D>0).*D);
    % The new iterate inverse orthogonal transform of WX
    X_iter=WT(WX);        
    % Gradient mapping
    d_h = X_old - X_iter;    
    % scale down the gradient mapping and image     
    D_H = d_h(id1) + d_h(id2) + d_h(id1+m) + d_h(id2+m);
    D_H = reshape(D_H/4,[M,N]);
    R = X_old(id1) + X_old(id2) + X_old(id1+m) + X_old(id2+m);        
    smallB_start = reshape(R/4,[M,N]);
    % Compute coarse correction conditions 
    nD_h = norm(X_old - X_lastcoarse,'fro');
    nGrad = norm(d_h,'fro');
    nD_H = norm(D_H,'fro')/nGrad;     
    if ((nD_H > kappa) && (nD_h > eta) && (nGrad> eta)),       
        % coarse error correction
        R = smooth_sqrt(smallB,smallB_start,lambda/2,smallPars,D_H,Ac_H,Ar_H,l);        
        d_h = kron(R - smallB_start,ones(2,2));
        tau = 1;        
        l = l+1;
        smooth_iter = [smooth_iter; iter];
    else
        % Gradient step  
        tau = 1/L;
        d_h = -2*Ac'*(Ac*X_old*Ar'-B)*Ar;          
    end;
    
        X_iter = X_old + tau*d_h;                
        WX=W(X_iter);        
        D=abs(WX)-lambda*tau;
        WX=sign(WX).*((D>0).*D);        
        X_iter=WT(WX);
        % compute function value
        t=sum(sum(abs(WX)));
        fun_val=norm(Ac*X_iter*Ar'-B,'fro')^2+lambda*t;    
    
    fun_all=[fun_all;fun_val];    
    if (fig>0)        
        figure(fig), imshow(X_iter,[]);        
    end
    fprintf('%4d  %10.5f  %6.3f  %6.3f',iter,fun_val,nD_H,nGrad);        
    if ((nD_H > kappa) && (nD_h > eta) && (nGrad > eta)),       
        fprintf(' >> coarse correction');
        X_lastcoarse = X_old;
    end;
    fprintf('\n');
    if fun_all(end)/fun_all(1) < epsilon,
        break;
    end;
end