function [X,f] = mista(inputImage,inputMethod,pars)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function provide top-level parameters for multilevel ISTA/FISTA to
% solve the composite convex optimisation. 
% 
% Based on the paper:
% - Panos Parpas, Duy V.N. Luong, Daniel Rueckert and Berc Rustem, 
% "A multilevel Proximal Algorithm for Large Scale Composite Convex Optimization" 
% Submitted to Mathematical Programming.
% -----------------------------------------------------------------------
% Further reference:
% - P.C. Hansen, J.G Nagy, and D.P. O'Leary, 
% "Deblurring images: Matrices, spectra and filtering",
% Fundamentals of Algorithms. SIAM, Philadelphia 2006.
% -----------------------------------------------------------------------
% Copyright (2014): 
% Panos Parpas, Duy V.N. Luong, Daniel Rueckert and Berc Rustem
%
% MISTA is distributed under the terms of
% the GNU General Public License 2.0.
%
% Permission to use, copy, modify, and distribute this software for
% any purpose without fee is hereby granted, provided that this entire
% notice is included in all copies of any software which is or includes
% a copy or modification of this software and in all copies of the
% supporting documentation for such software.
% This software is being provided "as is", without any express or
% implied warranty. In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose."
% -----------------------------------------------------------------------
% ----------------------------------------------------------------------- 
% INPUT
%
% inputImage    : Filename of the original image
% inputMethod   : 'ista' or 'fista'
% pars          : Parameters structure
% pars.MAXITER  : maximum number of iterations (Default = 50)
% pars.lambda   : regulariser parameter (1e-5)
% pars.wave     : orthogonal transformation (@dct)
% pars.iwave    : orthogonal transformation (@idct)
% pars.dim      : psf window ([16,16])
% pars.s        : psf standard deviation (8)
% pars.noisy    : Guassian Noise added to image (0.001)
% ** larger dim,s,noisy produces more corrupted images 
% pars.fig      : if fig>0 show image at each iteration, 0 otherwise (0)
% pars.BC       : Boundary Condition 'reflexive'(default) or 'periodic'
% pars.epsilon  : stopping condition, the percentage of F_k/F_0 (0.02)
% ** Without epsilon, program will run MAXITER iterations
% pars.kappa    : first condition to do coarse correction (0.49)
% pars.eta      : second condition to do coarse correction (1) 
% ** Smaller kappa & eta encourage more coarse correction
% -----------------------------------------------------------------------
% OUTPUT
% 
% X             : Solution of the problem min |A(X)-B|^2 + lambda |Wx|_1
% f             : history of objective functions

warning('off','all');
if ~exist('pars','var')   
    pars = [];
end
if ~isfield(pars,'lambda')    
    pars.lambda=1e-5;
end
if ~isfield(pars,'BC')    
    pars.BC='reflexive';
end
if ~isfield(pars,'MAXITER')
    pars.MAXITER= 50;
end
if ~isfield(pars,'kappa')
    pars.kappa = 0.49;
end
if ~isfield(pars,'eta')    
    pars.eta = 1;
end
if ~isfield(pars,'epsilon')    
    pars.epsilon = 0;
end
if ~isfield(pars,'wave')    
    pars.wave  = @dct;
    pars.iwave = @idct;
end;
if ~isfield(pars,'fig')    
    pars.fig=0;    
end;
if ~isfield(pars,'dim')
    pars.dim=[16,16];
end;
if ~isfield(pars,'s')
    pars.s= 8;
end;
if ~isfield(pars,'noisy')
    pars.noisy=0.001;
end;

% Read original image
X=double(imread(inputImage));
% scale down to image range \Omega = (0,1)
minX = min(X(:));
maxX = max(X(:));
X = (X - minX) / (maxX - minX);

% Generate Gaussian PSF parameter to blur image
[P,center]=psfGauss(pars.dim,pars.s);
% Blur image
B=imfilter(X,P,'symmetric');
randn('seed',314);
B= B + pars.noisy*randn(size(B));

scrsz = get(0,'ScreenSize');
if pars.fig >0,    
    figure(pars.fig), imshow(B,[]);    
end;

% function handle
fh = str2func(inputMethod);
[X,f,coarse_iters] = fh(B,P,center,pars.lambda,pars);

h1 = figure('Position',[1 scrsz(4)/4 scrsz(3)/4 scrsz(4)/2]);
figure(h1), subplot(1,2,1), imshow(B), subplot(1,2,2), imshow(X);

h2 = figure('Position',[100 scrsz(4)/3 scrsz(3)/5 scrsz(4)/2]);
figure(h2), scatter((1:length(f))',f,30,'b','fill');
figure(h2), hold, scatter(coarse_iters,f(coarse_iters),40,'r','fill'), hold;
legend('Gradient Update','Coarse Correction');      
figure(h2), hold, plot((1:length(f))',f,'--','linewidth',1),hold;
ylabel('F(x)');
xlabel('iterations');
title(['Objective value convergence of multilevel ',inputMethod]);
