function [X_iter,fun_all]=smooth_sqrt(B,B_start,lambda,pars,v,Ac,Ar,l)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function implements the coarse model of the general Multilevel FISTA 
% (see fista.m). It uses "square root" smoothing technique to smooth the 
% non-smooth regulariser. Other types of smoothing techniques are possible.
% This function performs computations at the coarse levels, and it calls
% itself obtain coarser corrections. 
% 
% Based on the paper:
% - Panos Parpas, Duy V.N. Luong, Daniel Rueckert and Berc Rustem, 
% "A multilevel Proximal Algorithm for Large Scale Composite Convex Optimization" 
% Submitted to Mathematical Programming.
% -----------------------------------------------------------------------
% Further references:
% - A. Beck and M. Teboulle. 
% "Smoothing and first order methods: A unified framework",
% SIAM Journal on Optimization, 22, 2012.
% -----------------------------------------------------------------------
% Copyright (2014): 
% Panos Parpas, Duy V.N. Luong, Daniel Rueckert and Berc Rustem
%
% MISTA is distributed under the terms of
% the GNU General Public License 2.0.
%
% Permission to use, copy, modify, and distribute this software for
% any purpose without fee is hereby granted, provided that this entire
% notice is included in all copies of any software which is or includes
% a copy or modification of this software and in all copies of the
% supporting documentation for such software.
% This software is being provided "as is", without any express or
% implied warranty. In particular, the authors do not make any
% representation or warranty of any kind concerning the merchantability
% of this software or its fitness for any particular purpose."
% -----------------------------------------------------------------------
% ----------------------------------------------------------------------- 
% INPUT
%
% B             : The coarse corrupted image
% B_start       : starting image for optimisation
% lambda        : Regularisation parameter
% pars          : Parameters structure
% pars.MAXITER  : maximum number of iterations (Default = 50)
% pars.wave     : orthogonal transformation (@dct)
% pars.iwave    : orthogonal transformation (@idct)
% pars.epsilon  : stopping condition, the changes |F_k - F_(k+1)| (0.01)
% v             : linear term for first order coherence (see Parpas et.al.)
% Ac,Ar         : column/row blurring operators
% l             : condition to do coarser levels
% -----------------------------------------------------------------------
% OUTPUT
% 
% X_iter        : Solution of the problem min |A(X)-B|^2 + lambda |Wx|_1
% fun_all       : History of function values at the finest resolution


% Assigning parameters according to pars and/or default values
flag = exist('pars','var');
if (flag && isfield(pars,'MAXITER'))
    MAXITER=pars.MAXITER;
else
    MAXITER=100;
end
if (flag && isfield(pars,'wave'))
    W=pars.wave;    
    WT=pars.iwave;    
else
    W=@dct;
    WT=@idct;
end
if (flag && isfield(pars,'mu'))
    mu=pars.mu;
else
    mu=0.2;
end
if (flag && isfield(pars,'epsilon'))
    epsilon=pars.epsilon;    
else
    epsilon = 0.02;
end
e = 0.01;
[m,n]=size(B);
%% initialisation
X_iter = B_start;
WX = W(X_iter);
tmp = sqrt(WX.^2+mu^2);
% Compute (A_H * x_H - B) by ---> Ac*x*Ar' - B
g = (Ac*X_iter*Ar') - B;
fun_val = norm(g,'fro')^2 + lambda*(sum(tmp(:))-m*n*mu);
% compute first coarse correction if level > 0
vterm = numel(v)>1;
if vterm,
    % Get the residual v term, for efficiency split to v1 and v2
    v1 = v - 2*g;
    v2 = -lambda*0.5*1./tmp;
    fun_all = fun_val;
    if l==0, % do coarser correction for this coarse model at first iter        
        M = m/2; N = n/2;
        R = ones(1,2);
        R = kron(speye(M,M),R);
        Ac_H = R*Ac*R'/2;
        Ar_H = R*Ar*R'/2;
        id1 = (1:2:m)';
        id2 = (2:2:m)';
        for i=1:(n/2)-1,
            id1 = [id1; (2*i*m+1:2:2*i*m+m)'];
            id2 = [id2; (2*i*m+2:2:2*i*m+m)'];    
        end;
        R = X_iter(id1) + X_iter(id2) + X_iter(id1+m) + X_iter(id2+m);    
        smallB_start = reshape(R/4,[M,N]);
        smallPars = [];        
        smallPars.epsilon = epsilon;           
        R = B(id1) + B(id2) + B(id1+m) + B(id2+m);            
        R = smooth_sqrt(reshape(R/4,[M,N]),smallB_start,lambda/2,smallPars,0,Ac_H,Ar_H,1);        
        X_iter = X_iter + kron(R-smallB_start,ones(2,2));
        WX=W(X_iter);
        tmp = sqrt(WX.^2+mu^2);        
        g = (Ac*X_iter*Ar') - B;        
        fun_val = norm(g,'fro')^2 + lambda*(sum(tmp(:))-m*n*mu);
        fun_all = [fun_all; fun_val];
        if fun_all(end)/fun_all(1)<epsilon,
          return;
        end;
    end;
else
    v1 = 0; v2 = 0;
    fun_all = fun_val;
end;
% Compute A_H' * ( A_H * x_H - B) = Ac'* tmp * Ar
g = Ac' * g * Ar;
for i=1:MAXITER    
    X_old=X_iter;  
    t = 1;
    g = -2*g-e*v1;
    normg = norm(g,'fro')^2;
    fun_val = fun_all(end) + 1000;
    while fun_val > fun_all(end) + 0.01*t*normg,
        % Gradient step for ||Ax-b||^2 
        X_iter=X_old + t*g;       
        % Gradient step for sqrt-smoothing of |x|           
        WX=W(X_iter);
        WX = WX - lambda/t*0.5*tmp - e/t*v2;
        X_iter = WT(WX);    
        tmp = sqrt(WX.^2+mu^2);
        g = (Ac*X_iter*Ar') - B;    
        fun_val=norm(g,'fro')^2 + lambda*(sum(tmp(:))-m*n*mu);
        t = t/2;
    end;
    fun_all=[fun_all;fun_val];        
    % gradient step A'(Ax-b)
    g = Ac'* g * Ar;
    if abs(fun_all(end-1)-fun_all(end))<epsilon,
        break;
    end;
end